@@ -4,6 +4,8 @@ test_that("seriesExtent works", {
   
   skip_on_cran()
   
+  skip_if_not_installed("sf")
+  
   x <- seriesExtent('Amador')
   
   # NULL result on network-related error
@@ -11,6 +13,8 @@ test_that("seriesExtent works", {
   
   expect_true(inherits(x, 'sf'))
   
+  skip_if_not_installed("terra")
+  
   x2 <- seriesExtent('Amador', type = 'raster')
   
   # NULL result on network-related error
